@@ -8,7 +8,7 @@
 # since each segment is represented as a point so it is necessary to
 # assign formatting to each point in the series.
 #
-# Copyright 2013, John McNamara, jmcnamara@cpan.org
+# Copyright 2013-2014, John McNamara, jmcnamara@cpan.org
 #
 import xlsxwriter
 
@@ -35,9 +35,8 @@ worksheet.write_column('B2', data[1])
 chart1 = workbook.add_chart({'type': 'pie'})
 
 # Configure the series. Note the use of the list syntax to define ranges:
-# List is [ sheet_name, first_row, first_col, last_row, last_col ].
 chart1.add_series({
-    'name': 'Pie sales data',
+    'name':       'Pie sales data',
     'categories': ['Sheet1', 1, 0, 3, 0],
     'values':     ['Sheet1', 1, 1, 3, 1],
 })
@@ -57,7 +56,7 @@ worksheet.insert_chart('C2', chart1, {'x_offset': 25, 'y_offset': 10})
 #
 
 # Create an example Pie chart like above.
-chart2 = workbook.add_chart({'type': 'pie', 'embedded': 1})
+chart2 = workbook.add_chart({'type': 'pie'})
 
 # Configure the series and add user defined segment colours.
 chart2.add_series({
