@@ -3,11 +3,11 @@
 # Example of cell locking and formula hiding in an Excel worksheet
 # using Python and the XlsxWriter module.
 #
-# Copyright 2013, John McNamara, jmcnamara@cpan.org
+# Copyright 2013-2014, John McNamara, jmcnamara@cpan.org
 #
-from xlsxwriter.workbook import Workbook
+import xlsxwriter
 
-workbook = Workbook('protection.xlsx')
+workbook = xlsxwriter.Workbook('protection.xlsx')
 worksheet = workbook.add_worksheet()
 
 # Create some cell formats with protection properties.
